﻿//-----------------------------------------------------------------------------
//
//  mainForm.cs
//
//  USB Performance Monitor Application
//  Copyright (C) 2010 Simon Inns
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Web:    http://www.waitingforfriday.com
//  Email:  simon.inns@gmail.com
//
//-----------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace USB_Performance_Monitor
    {
    public partial class mainForm : Form
        {
        public mainForm()
            {
            InitializeComponent();

            // Initialise the USB device class-library

            // Create the USB reference device object (passing VID and PID)
            theUsbPerformanceMeterDevice = new usbPerformanceMeterDevice(0x04D8, 0x0044);

            // Register for device change notifications
            theUsbPerformanceMeterDevice.registerForDeviceNotifications(this.Handle);

            // Add a listener for usb events
            theUsbPerformanceMeterDevice.usbEvent +=
                new usbPerformanceMeterDevice.usbEventsHandler(usbEvent_receiver);

            // Perform an initial search for the target device
            theUsbPerformanceMeterDevice.findTargetDevice();

            // Send the device settings
            theUsbPerformanceMeterDevice.updateDeviceSettings(Properties.Settings.Default.leftMeterSensitivity,
                Properties.Settings.Default.rightMeterSensitivity,
                Properties.Settings.Default.hddFadeOnSpeed * 100,
                Properties.Settings.Default.hddFadeOffSpeed * 100);

            // Initialise the button state tracker
            previousButtonState = false;

            // ----

            // Initialise the notify icon to notifyOff
            this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
            this.upmNotifyIcon.Visible = true;

            // Set up the performance counters
            this.leftMeterPerformanceCounter1 = new System.Diagnostics.PerformanceCounter();
            this.leftMeterPerformanceCounter2 = new System.Diagnostics.PerformanceCounter();
            this.rightMeterPerformanceCounter1 = new System.Diagnostics.PerformanceCounter();
            this.rightMeterPerformanceCounter2 = new System.Diagnostics.PerformanceCounter();
            this.hddLedReadPerformanceCounter = new System.Diagnostics.PerformanceCounter();
            this.hddLedWritePerformanceCounter = new System.Diagnostics.PerformanceCounter();

            // HDD LED read and write performance counters
            this.hddLedReadPerformanceCounter.CategoryName = "PhysicalDisk";
            this.hddLedReadPerformanceCounter.CounterName = "% Disk Read Time";
            this.hddLedReadPerformanceCounter.InstanceName = "_Total";

            this.hddLedWritePerformanceCounter.CategoryName = "PhysicalDisk";
            this.hddLedWritePerformanceCounter.CounterName = "% Disk Write Time";
            this.hddLedWritePerformanceCounter.InstanceName = "_Total";

            if (Properties.Settings.Default.hddLedEnabled == true) this.ledTargetLabel.Text = "Enabled";
            else this.ledTargetLabel.Text = "Disabled";

            // Left meter performance counters
            switch (Properties.Settings.Default.leftMeterFunction)
                {
                case "Processor":
                    this.leftMeterPerformanceCounter1.CategoryName = "Processor";
                    this.leftMeterPerformanceCounter1.CounterName = "% Processor Time";
                    this.leftMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    this.leftMeterTargetLabel.Text = "Processor utilisation";
                    break;
                case "Harddrive":
                    this.leftMeterPerformanceCounter1.CategoryName = "PhysicalDisk";
                    this.leftMeterPerformanceCounter1.CounterName = "% Disk Time";
                    this.leftMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    this.leftMeterTargetLabel.Text = "Hard drive utilisation";
                    break;
                case "Memory":
                    this.leftMeterPerformanceCounter1.CategoryName = "Memory";
                    this.leftMeterPerformanceCounter1.CounterName = "% Committed Bytes In Use";
                    this.leftMeterPerformanceCounter1.InstanceName = "";
                    this.leftMeterTargetLabel.Text = "Memory utilisation";
                    break;
                case "Network":
                    this.leftMeterPerformanceCounter1.CategoryName = "Network Interface";
                    this.leftMeterPerformanceCounter1.CounterName = "Current Bandwidth";
                    this.leftMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    this.leftMeterTargetLabel.Text = "Network utilisation";

                    this.leftMeterPerformanceCounter2.CategoryName = "Network Interface";
                    this.leftMeterPerformanceCounter2.CounterName = "Bytes Total/sec";
                    this.leftMeterPerformanceCounter2.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    break;
                case "Disabled":
                    this.leftMeterTargetLabel.Text = "Disabled";
                    break;
                }

            // Right meter performance counters
            switch (Properties.Settings.Default.rightMeterFunction)
                {
                case "Processor":
                    this.rightMeterPerformanceCounter1.CategoryName = "Processor";
                    this.rightMeterPerformanceCounter1.CounterName = "% Processor Time";
                    this.rightMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    this.rightMeterTargetLabel.Text = "Processor utilisation";
                    break;
                case "Harddrive":
                    this.rightMeterPerformanceCounter1.CategoryName = "PhysicalDisk";
                    this.rightMeterPerformanceCounter1.CounterName = "% Disk Time";
                    this.rightMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    this.rightMeterTargetLabel.Text = "Hard drive utilisation";
                    break;
                case "Memory":
                    this.rightMeterPerformanceCounter1.CategoryName = "Memory";
                    this.rightMeterPerformanceCounter1.CounterName = "% Committed Bytes In Use";
                    this.rightMeterPerformanceCounter1.InstanceName = "";
                    this.rightMeterTargetLabel.Text = "Memory utilisation";
                    break;
                case "Network":
                    this.rightMeterPerformanceCounter1.CategoryName = "Network Interface";
                    this.rightMeterPerformanceCounter1.CounterName = "Current Bandwidth";
                    this.rightMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    this.rightMeterTargetLabel.Text = "Network utilisation";

                    this.rightMeterPerformanceCounter2.CategoryName = "Network Interface";
                    this.rightMeterPerformanceCounter2.CounterName = "Bytes Total/sec";
                    this.rightMeterPerformanceCounter2.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    break;
                case "Disabled":
                    this.rightMeterTargetLabel.Text = "Disabled";
                    break;
                }

            // Update the form according to the user's settings
            updateFormFromSettings();

            // Set the closure flag (used for hiding window on close)
            closedByMenuSelection = false;
            }

        // Define the form's performance counters
        private System.Diagnostics.PerformanceCounter leftMeterPerformanceCounter1;
        private System.Diagnostics.PerformanceCounter leftMeterPerformanceCounter2;
        private System.Diagnostics.PerformanceCounter rightMeterPerformanceCounter1;
        private System.Diagnostics.PerformanceCounter rightMeterPerformanceCounter2;
        private System.Diagnostics.PerformanceCounter hddLedReadPerformanceCounter;
        private System.Diagnostics.PerformanceCounter hddLedWritePerformanceCounter;

        // Initialise the button state tracker flag
        private bool previousButtonState;

        // Create an instance of the USB performance meter
        private usbPerformanceMeterDevice theUsbPerformanceMeterDevice;

        // Callback method for WndProc
        protected override void WndProc(ref Message m)
            {
            // Pass the message to our message handler
            theUsbPerformanceMeterDevice.handleDeviceNotificationMessages(m);

            // Pass the message on to the base WndProc
            base.WndProc(ref m);
            }

        // Listener for USB events
        private void usbEvent_receiver(object o, EventArgs e)
            {
            // Check the status of the USB device and update the form accordingly
            if (theUsbPerformanceMeterDevice.isDeviceAttached)
                {
                // Device is attached, do tasks here
                this.usbStatusToolStripStatusLabel.Text = "USB Performance Meter Attached";

                // Send the device settings
                theUsbPerformanceMeterDevice.updateDeviceSettings(Properties.Settings.Default.leftMeterSensitivity,
                    Properties.Settings.Default.rightMeterSensitivity,
                    Properties.Settings.Default.hddFadeOnSpeed * 100,
                    Properties.Settings.Default.hddFadeOffSpeed * 100);
                }
            else
                {
                // Device is detached, do tasks here
                this.usbStatusToolStripStatusLabel.Text = "USB Performance Meter Not Detected";
                }
            }

        // Menu -> Tools -> Options selected
        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
            {
            Form optionsDialogue = new options();

            // Disable the main form update timer
            this.formUpdatetimer.Enabled = false;

            // Open the options dialogue
            optionsDialogue.ShowDialog();

            // Close the options dialogue
            optionsDialogue.Close();

            // Update the left meter performance counters
            switch (Properties.Settings.Default.leftMeterFunction)
                {
                case "Processor":
                    this.leftMeterPerformanceCounter1.CategoryName = "Processor";
                    this.leftMeterPerformanceCounter1.CounterName = "% Processor Time";
                    this.leftMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    this.leftMeterTargetLabel.Text = "Processor utilisation";
                    break;
                case "Harddrive":
                    this.leftMeterPerformanceCounter1.CategoryName = "PhysicalDisk";
                    this.leftMeterPerformanceCounter1.CounterName = "% Disk Time";
                    this.leftMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    this.leftMeterTargetLabel.Text = "Hard drive utilisation";
                    break;
                case "Memory":
                    this.leftMeterPerformanceCounter1.CategoryName = "Memory";
                    this.leftMeterPerformanceCounter1.CounterName = "% Committed Bytes In Use";
                    this.leftMeterPerformanceCounter1.InstanceName = "";
                    this.leftMeterTargetLabel.Text = "Memory utilisation";
                    break;
                case "Network":
                    this.leftMeterPerformanceCounter1.CategoryName = "Network Interface";
                    this.leftMeterPerformanceCounter1.CounterName = "Current Bandwidth";
                    this.leftMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    this.leftMeterTargetLabel.Text = "Network utilisation";

                    this.leftMeterPerformanceCounter2.CategoryName = "Network Interface";
                    this.leftMeterPerformanceCounter2.CounterName = "Bytes Total/sec";
                    this.leftMeterPerformanceCounter2.InstanceName = Properties.Settings.Default.leftMeterInstance;
                    break;
                case "Disabled":
                    this.leftMeterTargetLabel.Text = "Disabled";
                    break;
                }

            // Update the right meter performance counters
            switch (Properties.Settings.Default.rightMeterFunction)
                {
                case "Processor":
                    this.rightMeterPerformanceCounter1.CategoryName = "Processor";
                    this.rightMeterPerformanceCounter1.CounterName = "% Processor Time";
                    this.rightMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    this.rightMeterTargetLabel.Text = "Processor utilisation";
                    break;
                case "Harddrive":
                    this.rightMeterPerformanceCounter1.CategoryName = "PhysicalDisk";
                    this.rightMeterPerformanceCounter1.CounterName = "% Disk Time";
                    this.rightMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    this.rightMeterTargetLabel.Text = "Hard drive utilisation";
                    break;
                case "Memory":
                    this.rightMeterPerformanceCounter1.CategoryName = "Memory";
                    this.rightMeterPerformanceCounter1.CounterName = "% Committed Bytes In Use";
                    this.rightMeterPerformanceCounter1.InstanceName = "";
                    this.rightMeterTargetLabel.Text = "Memory utilisation";
                    break;
                case "Network":
                    this.rightMeterPerformanceCounter1.CategoryName = "Network Interface";
                    this.rightMeterPerformanceCounter1.CounterName = "Current Bandwidth";
                    this.rightMeterPerformanceCounter1.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    this.rightMeterTargetLabel.Text = "Network utilisation";

                    this.rightMeterPerformanceCounter2.CategoryName = "Network Interface";
                    this.rightMeterPerformanceCounter2.CounterName = "Bytes Total/sec";
                    this.rightMeterPerformanceCounter2.InstanceName = Properties.Settings.Default.rightMeterInstance;
                    break;
                case "Disabled":
                    this.rightMeterTargetLabel.Text = "Disabled";
                    break;
                }

            // Update the HDD LED status
            if (Properties.Settings.Default.hddLedEnabled == true) this.ledTargetLabel.Text = "Enabled";
            else this.ledTargetLabel.Text = "Disabled";

            // Send the device settings
            theUsbPerformanceMeterDevice.updateDeviceSettings(Properties.Settings.Default.leftMeterSensitivity * 100,
                Properties.Settings.Default.rightMeterSensitivity * 100,
                Properties.Settings.Default.hddFadeOnSpeed * 100,
                Properties.Settings.Default.hddFadeOffSpeed * 100);

            // Update the form settings and restart the timer
            updateFormFromSettings();
            }

        private bool closedByMenuSelection;

        // Menu -> File -> Exit selected
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
            {
            closedByMenuSelection = true;
            this.Close();
            }

        // Menu -> Help -> About selected
        private void aboutUPMToolStripMenuItem_Click(object sender, EventArgs e)
            {
            Form aboutUpmBox = new aboutBox();
            aboutUpmBox.ShowDialog();
            aboutUpmBox.Close();
            }

        // Form update timer tick
        private void formUpdatetimer_Tick(object sender, EventArgs e)
            {
            string ledColour = null;
            int diskReadTimePercentage = 0;
            int diskWriteTimePercentage = 0;

            // Update HDD LED
            if (Properties.Settings.Default.hddLedEnabled == true)
                {
                try
                    {
                    // Read the performance counter values
                    diskReadTimePercentage = Convert.ToInt32(hddLedReadPerformanceCounter.NextValue());
                    diskWriteTimePercentage = Convert.ToInt32(hddLedWritePerformanceCounter.NextValue());

                    // Range check
                    if (diskReadTimePercentage > 100) diskReadTimePercentage = 100;
                    if (diskWriteTimePercentage > 100) diskWriteTimePercentage = 100;
                    }
                catch
                    {
                    // If this fails (which it shouldn't, we just default the percentages to zero
                    diskReadTimePercentage = 0;
                    diskWriteTimePercentage = 0;
                    }

                // Get the correct colour for the LED
                if (diskReadTimePercentage > 0 && diskWriteTimePercentage == 0)
                    {
                    // HDD is reading
                    ledColour = Properties.Settings.Default.hddLedReadColour;
                    }
                else if (diskReadTimePercentage == 0 && diskWriteTimePercentage > 0)
                    {
                    // HDD is writing
                    ledColour = Properties.Settings.Default.hddLedWriteColour;
                    }
                else if (diskReadTimePercentage > 0 && diskWriteTimePercentage > 0)
                    {
                    // HDD is reading and writing
                    ledColour = Properties.Settings.Default.hddLedRwColour;
                    }
                else
                    {
                    // HDD is idle
                    ledColour = null;
                    }

                // Update the 'LED' on the form and the notify icon
                switch (ledColour)
                    {
                    case "Amber":
                        this.ledPictureBox.Image = Properties.Resources.amberOn;
                        if (Properties.Settings.Default.showHddLedStatusInTaskbar == true)
                            this.upmNotifyIcon.Icon = Properties.Resources.notifyAmber;
                        else this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    case "Blue":
                        this.ledPictureBox.Image = Properties.Resources.blueOn;
                        if (Properties.Settings.Default.showHddLedStatusInTaskbar == true)
                            this.upmNotifyIcon.Icon = Properties.Resources.notifyBlue;
                        else this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    case "Green":
                        this.ledPictureBox.Image = Properties.Resources.greenOn;
                        if (Properties.Settings.Default.showHddLedStatusInTaskbar == true)
                            this.upmNotifyIcon.Icon = Properties.Resources.notifyGreen;
                        else this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    case "Purple":
                        this.ledPictureBox.Image = Properties.Resources.purpleOn;
                        if (Properties.Settings.Default.showHddLedStatusInTaskbar == true)
                            this.upmNotifyIcon.Icon = Properties.Resources.notifyPurple;
                        else this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    case "Red":
                        this.ledPictureBox.Image = Properties.Resources.redOn;
                        if (Properties.Settings.Default.showHddLedStatusInTaskbar == true)
                            this.upmNotifyIcon.Icon = Properties.Resources.notifyRed;
                        else this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    case "White":
                        this.ledPictureBox.Image = Properties.Resources.whiteOn;
                        if (Properties.Settings.Default.showHddLedStatusInTaskbar == true)
                            this.upmNotifyIcon.Icon = Properties.Resources.notifyWhite;
                        else this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    case "Yellow":
                        this.ledPictureBox.Image = Properties.Resources.yellowOn;
                        if (Properties.Settings.Default.showHddLedStatusInTaskbar == true)
                            this.upmNotifyIcon.Icon = Properties.Resources.notifyYellow;
                        else this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    default:
                        this.ledPictureBox.Image = Properties.Resources.whiteOff;
                        this.upmNotifyIcon.Icon = Properties.Resources.notifyOff;
                        break;
                    }
                }
            else
                {
                // Set the form 'LED' to the off colour
                this.ledPictureBox.Image = Properties.Resources.whiteOff;
                }

            // Update left meter on form
            Int32 leftMeterPercentage = 0;
            switch (Properties.Settings.Default.leftMeterFunction)
                {
                case "Processor":
                    try
                        {
                        // Read the performance counter value
                        leftMeterPercentage = Convert.ToInt32(leftMeterPerformanceCounter1.NextValue());

                        // Range check
                        if (leftMeterPercentage > 100) leftMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default to disabled
                        leftMeterPercentage = -1;
                        }
                    break;
                case "Harddrive":
                    try
                        {
                        // Read the performance counter value
                        leftMeterPercentage = Convert.ToInt32(leftMeterPerformanceCounter1.NextValue());

                        // Range check
                        if (leftMeterPercentage > 100) leftMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default to disabled
                        leftMeterPercentage = -1;
                        }
                    break;
                case "Memory":
                    try
                        {
                        // Read the performance counter value
                        leftMeterPercentage = Convert.ToInt32(leftMeterPerformanceCounter1.NextValue());

                        // Range check
                        if (leftMeterPercentage > 100) leftMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default to disabled
                        leftMeterPercentage = -1;
                        }
                    break;
                case "Network":
                    try
                        {
                        Int32 bandwidth;
                        Int32 bytesSendAndReceievedSec;

                        // Read the performance counter values
                        bandwidth = Convert.ToInt32(leftMeterPerformanceCounter1.NextValue());
                        bytesSendAndReceievedSec = Convert.ToInt32(leftMeterPerformanceCounter2.NextValue());

                        // Check if the bandwidth is being overridden
                        if (Properties.Settings.Default.leftOverrideNetworkMaximumBandwith == true)
                            {
                            bandwidth = Convert.ToInt32(Properties.Settings.Default.leftOverrideNetworkMaximumBandwidthValue) * 1000000;
                            }

                        // Calculate utilisation
                        leftMeterPercentage = ((8 * bytesSendAndReceievedSec) / (bandwidth/100));

                        // Range check
                        if (leftMeterPercentage > 100) leftMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default the percentages to zero
                        leftMeterPercentage = 0;
                        }
                    break;
                case "Disabled":
                    // Left meter is disabled
                    leftMeterPercentage = -1;
                    break;
                }

            // Check the temporary disable setting for the left meter
            if (Properties.Settings.Default.leftMeterEnabled == false) leftMeterPercentage = -1;

            // Write the resulting percentage to the form
            if (leftMeterPercentage == -1)
                this.leftMeterValueLabel.Text = "Disabled";
            else this.leftMeterValueLabel.Text = String.Concat(Convert.ToString(leftMeterPercentage), "%");

            // Update right meter on form
            Int32 rightMeterPercentage = 0;
            switch (Properties.Settings.Default.rightMeterFunction)
                {
                case "Processor":
                    try
                        {
                        // Read the performance counter value
                        rightMeterPercentage = Convert.ToInt32(rightMeterPerformanceCounter1.NextValue());

                        // Range check
                        if (rightMeterPercentage > 100) rightMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default to disabled
                        rightMeterPercentage = -1;
                        }
                    break;
                case "Harddrive":
                    try
                        {
                        // Read the performance counter value
                        rightMeterPercentage = Convert.ToInt32(rightMeterPerformanceCounter1.NextValue());

                        // Range check
                        if (rightMeterPercentage > 100) rightMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default to disabled
                        rightMeterPercentage = -1;
                        }
                    break;
                case "Memory":
                    try
                        {
                        // Read the performance counter value
                        rightMeterPercentage = Convert.ToInt32(rightMeterPerformanceCounter1.NextValue());

                        // Range check
                        if (rightMeterPercentage > 100) rightMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default to disabled
                        rightMeterPercentage = -1;
                        }
                    break;
                case "Network":
                    try
                        {
                        Int32 bandwidth;
                        Int32 bytesSendAndReceievedSec;

                        // Read the performance counter values
                        bandwidth = Convert.ToInt32(rightMeterPerformanceCounter1.NextValue());
                        bytesSendAndReceievedSec = Convert.ToInt32(rightMeterPerformanceCounter2.NextValue());

                        // Check if the bandwidth is being overridden
                        if (Properties.Settings.Default.rightOverrideNetworkMaximumBandwith == true)
                            {
                            bandwidth = Convert.ToInt32(Properties.Settings.Default.rightOverrideNetworkMaximumBandwidthValue) * 1000000;
                            }

                        // Calculate utilisation
                        rightMeterPercentage = ((8 * bytesSendAndReceievedSec) / (bandwidth / 100));

                        // Range check
                        if (rightMeterPercentage > 100) rightMeterPercentage = 100;
                        }
                    catch
                        {
                        // If this fails we just default the percentages to zero
                        rightMeterPercentage = 0;
                        }
                    break;
                case "Disabled":
                    // Right meter is disabled
                    rightMeterPercentage = -1;
                    break;
                }

            // Check the temporary disable setting for the right meter
            if (Properties.Settings.Default.rightMeterEnabled == false) rightMeterPercentage = -1;

            // Write the resulting percentage to the form
            if (rightMeterPercentage == -1)
                this.rightMeterValueLabel.Text = "Disabled";
            else this.rightMeterValueLabel.Text = String.Concat(Convert.ToString(rightMeterPercentage), "%");

            // Update the USB device
            int usbLeftMeterPercentage;
            int usbRightMeterPercentage;
            int usbLedRedValue;
            int usbLedGreenValue;
            int usbLedBlueValue;

            // Meters
            if (leftMeterPercentage != -1) usbLeftMeterPercentage = leftMeterPercentage;
            else usbLeftMeterPercentage = 0;

            if (rightMeterPercentage != -1) usbRightMeterPercentage = rightMeterPercentage;
            else usbRightMeterPercentage = 0;

            // LED colour
            switch (ledColour)
                {
                case "Amber":
                    usbLedRedValue = 255;
                    usbLedGreenValue = 60;
                    usbLedBlueValue = 0;
                    break;
                case "Blue":
                    usbLedRedValue = 0;
                    usbLedGreenValue = 0;
                    usbLedBlueValue = 255;
                    break;
                case "Green":
                    usbLedRedValue = 0;
                    usbLedGreenValue = 255;
                    usbLedBlueValue = 0;
                    break;
                case "Purple":
                    usbLedRedValue = 109;
                    usbLedGreenValue = 11;
                    usbLedBlueValue = 230;
                    break;
                case "Red":
                    usbLedRedValue = 255;
                    usbLedGreenValue = 0;
                    usbLedBlueValue = 0;
                    break;
                case "White":
                    usbLedRedValue = 255;
                    usbLedGreenValue = 255;
                    usbLedBlueValue = 255;
                    break;
                case "Yellow":
                    usbLedRedValue = 255;
                    usbLedGreenValue = 120;
                    usbLedBlueValue = 0;
                    break;
                default:
                    // LED is off
                    usbLedRedValue = 0;
                    usbLedGreenValue = 0;
                    usbLedBlueValue = 0;
                    break;
                }

            // Adjust the RGB intensity
            if (Properties.Settings.Default.hddIncreaseBrightnessAccordingToUtilisation == true)
                {
                // Check what we need to base the intensity on
                if (diskReadTimePercentage > 0 && diskWriteTimePercentage > 0)
                    {
                    // Read/Write
                    int percentage = (diskWriteTimePercentage + diskReadTimePercentage) / 2;
                    usbLedRedValue = (usbLedRedValue / 100) * percentage;
                    usbLedGreenValue = (usbLedGreenValue / 100) * percentage;
                    usbLedBlueValue = (usbLedBlueValue / 100) * percentage;
                    }
                else if (diskReadTimePercentage == 0 && diskWriteTimePercentage > 0)
                    {
                    // Write
                    usbLedRedValue = (usbLedRedValue / 100) * diskWriteTimePercentage;
                    usbLedGreenValue = (usbLedGreenValue / 100) * diskWriteTimePercentage;
                    usbLedBlueValue = (usbLedBlueValue / 100) * diskWriteTimePercentage;
                    }
                else
                    {
                    // Read
                    usbLedRedValue = (usbLedRedValue / 100) * diskReadTimePercentage;
                    usbLedGreenValue = (usbLedGreenValue / 100) * diskReadTimePercentage;
                    usbLedBlueValue = (usbLedBlueValue / 100) * diskReadTimePercentage;
                    }
                }

            // Write the changes to the device
            bool detachUsbOnButton = false;
            if (String.Compare(Properties.Settings.Default.buttonFunction, "Usb") == 0) detachUsbOnButton = true;

            bool buttonState = false;
            buttonState = theUsbPerformanceMeterDevice.updateDevice(usbLeftMeterPercentage, usbRightMeterPercentage,
                usbLedRedValue, usbLedGreenValue, usbLedBlueValue, detachUsbOnButton);

            // Stop the button from 'auto repeating'
            if (buttonState == true && previousButtonState == true) buttonState = false;
            else if (buttonState == false && previousButtonState == true) previousButtonState = false;

            // Check the button state
            if (buttonState)
                {
                previousButtonState = true;

                // Button is currently pressed

                // Note: no changes to settings made using the button persist
                switch (Properties.Settings.Default.buttonFunction)
                    {
                    case "Led":
                        // Enable/disable HDD LED
                        if (Properties.Settings.Default.hddLedEnabled == true)
                            Properties.Settings.Default.hddLedEnabled = false;
                        else Properties.Settings.Default.hddLedEnabled = true;

                        // Update the HDD LED status
                        if (Properties.Settings.Default.hddLedEnabled == true) this.ledTargetLabel.Text = "Enabled";
                        else this.ledTargetLabel.Text = "Disabled";
                        break;

                    case "Meters":
                        if (Properties.Settings.Default.leftMeterEnabled == true || Properties.Settings.Default.rightMeterEnabled == true)
                            {
                            // Disable the meters
                            Properties.Settings.Default.leftMeterEnabled = false;
                            Properties.Settings.Default.rightMeterEnabled = false;
                            }
                        else
                            {
                            // Enable the meters
                            Properties.Settings.Default.leftMeterEnabled = true;
                            Properties.Settings.Default.rightMeterEnabled = true;
                            }
                        break;

                    case "MetersAndLed":
                        if (Properties.Settings.Default.leftMeterEnabled == true || Properties.Settings.Default.rightMeterEnabled == true ||
                            Properties.Settings.Default.hddLedEnabled == true)
                            {
                            // Disable the meters
                            Properties.Settings.Default.leftMeterEnabled = false;
                            Properties.Settings.Default.rightMeterEnabled = false;
                            Properties.Settings.Default.hddLedEnabled = false;
                            }
                        else
                            {
                            // Enable the meters
                            Properties.Settings.Default.leftMeterEnabled = true;
                            Properties.Settings.Default.rightMeterEnabled = true;
                            Properties.Settings.Default.hddLedEnabled = true;
                            }

                        // Update the HDD LED status
                        if (Properties.Settings.Default.hddLedEnabled == true) this.ledTargetLabel.Text = "Enabled";
                        else this.ledTargetLabel.Text = "Disabled";
                        break;

                    case "Usb":
                        // This case is handled by the USB device, nothing for us to do here
                        break;

                    case "None":
                        // Nothing to do here
                        break;
                    }
                }
            }

        // Update the form according to the user's settings
        private void updateFormFromSettings()
            {
            // Set the form update timer interval
            this.formUpdatetimer.Interval = Properties.Settings.Default.meterUpdateSpeed * 100;

            // Start the form update timer
            this.formUpdatetimer.Enabled = true;
            }

        // Notifiy icon has been double-clicked
        private void upmNotifyIcon_MouseDoubleClick(object sender, MouseEventArgs e)
            {
            // Show the main window
            this.Show();
            WindowState = FormWindowState.Normal;
            }

        // Menu -> Options -> Hide in taskbar selected
        private void hideInTaskbarToolStripMenuItem_Click(object sender, EventArgs e)
            {
            this.Hide();
            }

        // Main form loaded event
        private void mainForm_Load(object sender, EventArgs e)
            {
            // Check the hide in taskbar on startup setting
            if (Properties.Settings.Default.hideInTaskbarOnStartUp == true)
                {
                BeginInvoke(new MethodInvoker(delegate
                {
                    this.Hide();
                }));
                }
            else
                {
                // Show the main window
                this.Show();
                WindowState = FormWindowState.Normal;
                }
            }

        // Main form closing event
        private void mainForm_Closing(object sender, FormClosingEventArgs e)
            {
            // Should we hide on close?
            if (Properties.Settings.Default.hideInTaskbarWhenCloseIconClicked == true && closedByMenuSelection == false)
                {
                this.Hide();
                e.Cancel = true;
                }
            else
                {
                // Disable the form update and set the device to zero with the LED off
                this.formUpdatetimer.Enabled = false;

                theUsbPerformanceMeterDevice.updateDevice(0, 0, 0, 0, 0, false);
                }
            }

        // Resize event on main form
        private void mainForm_Resize(object sender, EventArgs e)
            {
            // If hide on minimise is selected hide the windows (if the resize event is a minimise)
            if (Properties.Settings.Default.hideIntaskbarWhenMinimised == true)
                {
                if (FormWindowState.Minimized == WindowState)
                    this.Hide();
                }
            }
        }
    }
